<?php

namespace FiloBlu\Refilo\Model\Consumer;

use FiloBlu\Esb\Api\Data\MessageInterface;
use FiloBlu\Esb\Api\Data\MessageInterfaceFactory;
use FiloBlu\Esb\Api\Data\ObjectTypeDescriptorInterface;
use FiloBlu\Esb\Api\Data\StatusInterface;
use FiloBlu\Esb\Api\Data\StatusInterfaceFactory;
use FiloBlu\Esb\Core\Expression\EvaluatorInterfaceFactory;
use FiloBlu\Esb\Core\Extractor\ObjectTypeFromMessage;
use FiloBlu\Esb\Framework\Consumer\AbstractConsumer;
use FiloBlu\Refilo\Model\NewsletterManagementFactory;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;

/**
 * Class FormConsumer
 * @package FiloBlu\Refilo\Model\Consumer
 */
class NewsletterConsumer extends AbstractConsumer
{

    /**
     * @var NewsletterManagementFactory
     */
    private $newsletterFactory;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * NewsletterConsumer constructor.
     * @param EvaluatorInterfaceFactory $evaluatorFactory
     * @param ObjectTypeDescriptorInterface $objectTypeDescriptor
     * @param MessageInterfaceFactory $messageFactory
     * @param ObjectTypeFromMessage $objectTypeFromMessage
     * @param StatusInterfaceFactory $statusFactory
     * @param StoreManagerInterface $storeManager
     * @param LoggerInterface $logger
     * @param NewsletterManagementFactory $newsletter
     */
    public function __construct(
        EvaluatorInterfaceFactory $evaluatorFactory,
        ObjectTypeDescriptorInterface $objectTypeDescriptor,
        MessageInterfaceFactory $messageFactory,
        ObjectTypeFromMessage $objectTypeFromMessage,
        StatusInterfaceFactory $statusFactory,
        StoreManagerInterface $storeManager,
        LoggerInterface $logger,
        NewsletterManagementFactory $newsletter
    ) {
        parent::__construct($evaluatorFactory, $objectTypeDescriptor, $messageFactory, $objectTypeFromMessage, $statusFactory, $logger);
        $this->newsletterFactory = $newsletter;
        $this->storeManager = $storeManager;
    }

    /**
     * @param MessageInterface $message
     * @return MessageInterface
     */
    public function consume(MessageInterface $message): MessageInterface
    {
        $newsletterModel = $this->newsletterFactory->create();
        $payload = $message->getPayload();

        $result = $newsletterModel->subscribe(
            $payload->getDataByPath('fields/email'),
            $payload->getDataByPath('fields/source'),
            $payload->getDataByPath('fields/firstname'),
            $payload->getDataByPath('fields/lastname'),
            $payload->getDataByPath('fields/dob'),
            $payload->getDataByPath('fields/country'),
            $payload->getDataByPath('fields/gender'),
            $payload->getDataByPath('fields/collections'),
            $payload->getDataByPath('fields/language'),
            $payload->getDataByPath('fields/website_name'),
            $payload->getDataByPath('fields/phone'),
            $payload->getDataByPath('fields/store')
        );

        $this->getStatus()->setOutputData($result->toJson())->setCode(StatusInterface::SUCCESS);
        return $this->messageFactory->create();
    }

    public function getName(): string
    {
        return 'Refilo Form Newsletter consumer';
    }

    public function getUniqueName(): string
    {
        return 'FiloBlu_Refilo::form_newsletter_consumer';
    }
}
